/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.editor;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IPersistableElement;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.api.service.IClassificationService;
import eu.etaxonomy.cdm.api.service.ITaxonNodeService;
import eu.etaxonomy.cdm.api.service.ITaxonService;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.cdm.model.taxon.ITaxonTreeNode;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.DataChangeBridge;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;


/**
 * <p>TaxonEditorInput class.</p>
 *
 * @author n.hoffmann
 * @created 19.03.2009
 * @version 1.0
 */
public class TaxonEditorInput  extends CdmEntitySessionInput implements IEditorInput, IConversationEnabled, IPersistableElement {

    private static final String INCORRECT_STATE = Messages.TaxonEditorInput_INCORRECT_STATE;

    private final ConversationHolder conversation;

    private TaxonNode taxonNode;

    private TaxonEditorInputDataChangeBehaviour dataChangeBehavior;

    private TaxonBase initiallySelectedTaxonBase;

    private enum CdmType {
        TAXON_NODE,
        TAXON_BASE,
        PARENT_TAXON_NODE
    }

    private TaxonEditorInput(UUID uuid, CdmType type) {
        super(true);
        this.conversation = CdmStore.createConversation();
        switch(type) {
        case PARENT_TAXON_NODE:
            initForParentTaxonNode(uuid);
            break;
        case TAXON_BASE:
            initForTaxonBase(uuid);
            break;
        case TAXON_NODE:
            initForTaxonNode(uuid);
            break;
        }
    }

    private void init(TaxonNode taxonNode) {
        this.taxonNode = taxonNode;
    }


    /**
     * <p>NewInstance</p>
     *
     * @param taxonNodeUuid a {@link java.util.UUID} object.
     * @return a {@link eu.etaxonomy.taxeditor.editor.TaxonEditorInput} object.
     * @throws java.lang.Exception if any.
     */
    private void initForTaxonNode(UUID taxonNodeUuid) {


        TaxonNode taxonNode = CdmStore.getService(ITaxonNodeService.class).load(taxonNodeUuid, getTaxonNodePropertyPaths());

        if(taxonNode == null){
            MessagingUtils.warningDialog(Messages.TaxonEditorInput_NOT_IMPLEMENTED, TaxonEditorInput.class, Messages.TaxonEditorInput_NOT_IMPLEMENTED_MESSAGE);
        }
        init(taxonNode);

    }

    private void initForTaxonBase(UUID taxonBaseUuid) {
        TaxonBase taxonBase = CdmStore.getService(ITaxonService.class).load(taxonBaseUuid, getTaxonBasePropertyPaths());
        if (taxonBase != null){
            if(taxonBase.isInstanceOf(Taxon.class)){
                Taxon taxon = CdmBase.deproxy(taxonBase, Taxon.class);

                if (taxon.getTaxonNodes().size() == 0 && taxon.isMisapplication()){
                    // TODO get accepted taxon
                    MessagingUtils.info(Messages.TaxonEditorInput_OPEN_MISSAPPLIED_NAME);

                    Set<Taxon> acceptedTaxa = new HashSet<Taxon>();
                    Set<TaxonRelationship> relations = taxon.getRelationsFromThisTaxon();
                    for(TaxonRelationship relation : relations){
                        if(relation.getType().equals(TaxonRelationshipType.MISAPPLIED_NAME_FOR())){
                            acceptedTaxa.add(relation.getToTaxon());
                        }
                    }
                    setInputForMultipleTaxa(conversation, acceptedTaxa);

                }else{
                    setInputForMultipleNodes(conversation, taxon.getTaxonNodes());
                }
            }else if(taxonBase instanceof Synonym){
                Synonym synonym = (Synonym) taxonBase;

                Set<Taxon> taxa = new HashSet<>();
                Taxon taxon = synonym.getAcceptedTaxon();
                if (taxon != null){
                	taxa.add(taxon);
                }
                setInputForMultipleTaxa(conversation, taxa);
            }
        }
    }


    /**
     * <p>NewEmptyInstance</p>
     *
     * @param parentNodeUuid a {@link java.util.UUID} object.
     * @return a {@link eu.etaxonomy.taxeditor.editor.TaxonEditorInput} object.
     */
    private void initForParentTaxonNode(UUID parentNodeUuid){


        TaxonName name = PreferencesUtil.getPreferredNomenclaturalCode().getNewTaxonNameInstance(null);
        ITaxonTreeNode parentNode = CdmStore.getService(IClassificationService.class).getTreeNodeByUuid(parentNodeUuid);

        Taxon newTaxon = Taxon.NewInstance(name, parentNode.getReference());
        TaxonNode newTaxonNode = parentNode.addChildTaxon(newTaxon, parentNode.getReference(), parentNode.getMicroReference());

        // add the new taxon to the editors persistence context
        UUID newTaxonNodeUuid = CdmStore.getService(ITaxonNodeService.class).save(newTaxonNode).getUuid();

        initForTaxonNode(newTaxonNodeUuid);
    }




    private void setInputForMultipleNodes(ConversationHolder conversation, Set<TaxonNode> taxonNodes){
        if(taxonNodes.size() == 1){
            TaxonNode taxonNode = taxonNodes.iterator().next();
            init(taxonNode);
        }else if(taxonNodes.size() > 1){
            TaxonNode taxonNode = ChooseFromMultipleTaxonNodesDialog.choose(taxonNodes);
            if(taxonNode != null){
                init(taxonNode);
            }
        } else if (taxonNodes.size() == 0) {
            // this is an undesired state
            MessagingUtils.warningDialog(INCORRECT_STATE,TaxonEditorInput.class,Messages.TaxonEditorInput_TAXON_NOT_IN_CLASSIFICATION);
        }
    }

    private void setInputForMultipleTaxa(ConversationHolder conversation, Set<Taxon> taxa){
        if(taxa.size() == 1){
            Taxon taxon = taxa.iterator().next();
            Set<TaxonNode> nodes = taxon.getTaxonNodes();
            setInputForMultipleNodes(conversation, nodes);
        }else if(taxa.size() > 1){
            Set<TaxonNode> taxonNodes = new HashSet<TaxonNode>();
            for ( Taxon taxon : taxa ){
                taxonNodes.addAll(taxon.getTaxonNodes());
            }
            setInputForMultipleNodes(conversation, taxonNodes);
        }else if(taxa.size() == 0){
            // this is an undesired state
            MessagingUtils.warningDialog(INCORRECT_STATE, TaxonEditorInput.class, Messages.TaxonEditorInput_NO_ACCEPTED_TAXON_PRESENT);
        }
    }

    /**
     * <p>NewInstance</p>
     *
     * @param taxonNodeUuid a {@link java.util.UUID} object.
     * @return a {@link eu.etaxonomy.taxeditor.editor.TaxonEditorInput} object.
     * @throws java.lang.Exception if any.
     */
    public static TaxonEditorInput NewInstance(UUID taxonNodeUuid) throws Exception {
        return new TaxonEditorInput(taxonNodeUuid, CdmType.TAXON_NODE);

    }

    /**
     * <p>NewInstanceFromTaxonBase</p>
     *
     * @param taxonBaseUuid a {@link java.util.UUID} object.
     * @return a {@link eu.etaxonomy.taxeditor.editor.TaxonEditorInput} object.
     */
    public static TaxonEditorInput NewInstanceFromTaxonBase(UUID taxonBaseUuid){
        return new TaxonEditorInput(taxonBaseUuid, CdmType.TAXON_BASE);
    }


    /**
     * <p>NewEmptyInstance</p>
     *
     * @param parentNodeUuid a {@link java.util.UUID} object.
     * @return a {@link eu.etaxonomy.taxeditor.editor.TaxonEditorInput} object.
     */
    public static TaxonEditorInput NewEmptyInstance(UUID parentNodeUuid){
        return new TaxonEditorInput(parentNodeUuid, CdmType.PARENT_TAXON_NODE);
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IEditorInput#exists()
     */
    /**
     * <p>exists</p>
     *
     * @return a boolean.
     */
    @Override
    public boolean exists() {
        return taxonNode != null;
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IEditorInput#getImageDescriptor()
     */
    /**
     * <p>getImageDescriptor</p>
     *
     * @return a {@link org.eclipse.jface.resource.ImageDescriptor} object.
     */
    @Override
    public ImageDescriptor getImageDescriptor() {
        return null;
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IEditorInput#getName()
     */
    /**
     * <p>getName</p>
     *
     * @return a {@link java.lang.String} object.
     */
    @Override
    public String getName() {
        if(getTaxon() == null){
            return null;
        }
        TaxonName name = getTaxon().getName();
        if (name == null || name.getTitleCache() == null) {
            return Messages.TaxonEditorInput_NEW_TAXON;
        } else {
            return name.getTitleCache();
        }
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IEditorInput#getPersistable()
     */
    /**
     * <p>getPersistable</p>
     *
     * @return a {@link org.eclipse.ui.IPersistableElement} object.
     */
    @Override
    public IPersistableElement getPersistable() {
        //		if(CdmStore.isActive()){
        //			TaxonNode test = CdmStore.getTaxonTreeService().getTaxonNodeByUuid(taxonNode.getUuid());
        //			boolean isPersistable = CdmStore.getTaxonTreeService().getTaxonNodeByUuid(taxonNode.getUuid()) != null;
        //			if (isPersistable) {
        //				return this;
        //			}
        //		}
        return null;
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IEditorInput#getToolTipText()
     */
    /**
     * <p>getToolTipText</p>
     *
     * @return a {@link java.lang.String} object.
     */
    @Override
    public String getToolTipText() {
        return getName();
    }

    /* (non-Javadoc)
     * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
     */
    /** {@inheritDoc} */
    @Override
    public Object getAdapter(Class adapter) {

        if (adapter == Taxon.class) {
            return getTaxon();
        }

        if (adapter == TaxonNode.class) {
            return taxonNode;
        }

        return null;
    }

    /**
     * {@inheritDoc}
     *
     * Overrides equals to ensure that a taxon can only be edited by
     * one editor at a time.
     */
    @Override
    public boolean equals(Object obj) {
        if (TaxonEditorInput.class.equals(obj.getClass())
                && getTaxon() != null
                && getTaxon().equals(((TaxonEditorInput) obj).getTaxon())) {
            if (((TaxonEditorInput) obj).getInitiallySelectedTaxonBase() != null){
                setInitiallySelectedTaxonBase(((TaxonEditorInput) obj).getInitiallySelectedTaxonBase());
            }
            return true;
        }
        return false;
    }

    /**
     * <p>getTaxon</p>
     *
     * @return the taxon
     */
    public Taxon getTaxon(){
        Taxon taxon = CdmBase.deproxy(taxonNode.getTaxon(), Taxon.class);
        return taxon;
    }

    /**
     * <p>Getter for the field <code>taxonNode</code>.</p>
     *
     * @return the taxonNode
     */
    public TaxonNode getTaxonNode() {
        return taxonNode;
    }

    /*
     * (non-Javadoc)
     * @see eu.etaxonomy.cdm.api.conversation.IConversationEnabled#getConversationHolder()
     */
    /**
     * <p>getConversationHolder</p>
     *
     * @return a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
     */
    @Override
    public ConversationHolder getConversationHolder() {
        return conversation;
    }

    /*
     * (non-Javadoc)
     * @see eu.etaxonomy.cdm.persistence.hibernate.ICdmPostCrudObserver#update(eu.etaxonomy.cdm.persistence.hibernate.CdmCrudEvent)
     */
    /** {@inheritDoc} */
    @Override
    public void update(CdmDataChangeMap events) {
        if(dataChangeBehavior == null){
            dataChangeBehavior = new TaxonEditorInputDataChangeBehaviour(this);
        }

        DataChangeBridge.handleDataChange(events, dataChangeBehavior);
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IPersistableElement#getFactoryId()
     */
    /**
     * <p>getFactoryId</p>
     *
     * @return a {@link java.lang.String} object.
     */
    @Override
    public String getFactoryId() {
        return TaxonEditorInputFactory.getFactoryId();
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.IPersistable#saveState(org.eclipse.ui.IMemento)
     */
    /** {@inheritDoc} */
    @Override
    public void saveState(IMemento memento) {
        TaxonEditorInputFactory.saveState(memento, this);
    }


    /**
     * <p>Setter for the field <code>initiallySelectedTaxonBase</code>.</p>
     *
     * @param taxonBase a {@link eu.etaxonomy.cdm.model.taxon.TaxonBase} object.
     */
    public void setInitiallySelectedTaxonBase(TaxonBase taxonBase) {
        this.initiallySelectedTaxonBase = taxonBase;
    }

    /**
     * <p>Getter for the field <code>initiallySelectedTaxonBase</code>.</p>
     *
     * @return a {@link eu.etaxonomy.cdm.model.taxon.TaxonBase} object.
     */
    public TaxonBase getInitiallySelectedTaxonBase() {
        return initiallySelectedTaxonBase;
    }

//    @Override
//    public String toString() {
//        return String.format("%s[%s]", this.getClass().getSimpleName(), getTaxon());
//    }


    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled#getRootEntities()
     */
    @Override
    public List<TaxonNode> getRootEntities() {
        return Arrays.asList(taxonNode);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.editor.CdmEntitySessionInput#merge()
     */
    @Override
    public void merge() {
       CdmStore.getService(ITaxonNodeService.class).merge(taxonNode, true);

    }

    @Override
    public Map<Object, List<String>> getPropertyPathsMap() {
        return null;
    }

    private List<String> getTaxonNodePropertyPaths() {
        List<String> taxonNodePropertyPaths = new ArrayList<String>();
        for(String propertyPath : getTaxonBasePropertyPaths()) {
            taxonNodePropertyPaths.add("taxon." + propertyPath); //$NON-NLS-1$
        }
        return taxonNodePropertyPaths;
    }

    private List<String> getTaxonBasePropertyPaths() {
        List<String> taxonBasePropertyPaths = Arrays.asList(new String[] {
                "sec", //$NON-NLS-1$
                "createdBy", //$NON-NLS-1$
                "updatedBy", //$NON-NLS-1$
                "annotations", //$NON-NLS-1$
                "markers", //$NON-NLS-1$
                "credits", //$NON-NLS-1$
                "extensions", //$NON-NLS-1$
                "rights", //$NON-NLS-1$
                "sources", //$NON-NLS-1$
                "descriptions", //$NON-NLS-1$
                "relationsToThisTaxon", //$NON-NLS-1$
                "relationsFromThisTaxon", //$NON-NLS-1$
                "taxonNodes", //$NON-NLS-1$
                "descriptions.descriptionElements.feature", //$NON-NLS-1$
                "descriptions.descriptionElements.area", //$NON-NLS-1$
                "descriptions.descriptionElements.status", //$NON-NLS-1$
                "descriptions.markers", //$NON-NLS-1$
                "name.descriptions", //$NON-NLS-1$
                "name.typeDesignations", //$NON-NLS-1$
                "name.status", //$NON-NLS-1$
                "name.nomenclaturalReference.inReference", //$NON-NLS-1$
                "name.taxonBases.taxonNodes", //$NON-NLS-1$
                "name.relationsFromThisName", //$NON-NLS-1$
                "name.relationsToThisName", //$NON-NLS-1$
                "name.homotypicalGroup.typifiedNames.taxonBases.synonymRelations.synonym.name.status", //$NON-NLS-1$
                "name.homotypicalGroup.typifiedNames.relationsToThisName.fromName", //$NON-NLS-1$
                "synonymRelations.synonym.name.status.type", //$NON-NLS-1$
                "synonymRelations.synonym.name.relationsToThisName.fromName", //$NON-NLS-1$
                "synonymRelations.synonym.name.nomenclaturalReference.inReference.authorship", //$NON-NLS-1$
                "synonymRelations.synonym.name.nomenclaturalReference.authorship", //$NON-NLS-1$
                "synonymRelations.synonym.name.homotypicalGroup.typifiedNames.taxonBases.synonymRelations" //$NON-NLS-1$
        });

        return taxonBasePropertyPaths;
    }

}
