/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.operation;

import java.util.Set;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.api.service.ITaxonService;
import eu.etaxonomy.cdm.api.service.UpdateResult;
import eu.etaxonomy.cdm.api.service.exception.HomotypicalGroupChangeException;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.cdm.model.taxon.ITaxonTreeNode;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.operation.AbstractPersistentPostOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * Change the taxonomic parent of a given taxon.
 *
 * @author n.hoffmann
 * @created 16.01.2009
 */
public class ChangeSynonymToAcceptedTaxonOperation extends AbstractPersistentPostOperation {

	private final Synonym synonym;
	private final ITaxonTreeNode parentNode;

	private TaxonNode newNode;

	//private final Set<TaxonName> namesInHomotypicGroup;

	/**
	 * <p>Constructor for ChangeSynonymToAcceptedTaxonOperation.</p>
	 *
	 * @param label a {@link java.lang.String} object.
	 * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
	 * @param taxon a {@link eu.etaxonomy.cdm.model.taxon.Taxon} object.
	 * @param parentNode a {@link eu.etaxonomy.cdm.model.taxon.ITaxonTreeNode} object.
	 * @param synonym a {@link eu.etaxonomy.cdm.model.taxon.Synonym} object.
	 * @param synonymsInHomotypicalGroup an array of {@link eu.etaxonomy.cdm.model.taxon.Synonym} objects.
	 * @param postOperationEnabled a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
	 * @param conversationEnabled a {@link eu.etaxonomy.cdm.api.conversation.IConversationEnabled} object.
	 */
	public ChangeSynonymToAcceptedTaxonOperation(String label,
	        IUndoContext undoContext,
			Taxon taxon,
			ITaxonTreeNode parentNode,
			Synonym synonym,
			Set<TaxonName> namesInHomotypicalGroup,
			IPostOperationEnabled postOperationEnabled,
			IConversationEnabled conversationEnabled,
			ICdmEntitySessionEnabled cdmEntitySessionEnabled) {
		super(label, undoContext, postOperationEnabled, conversationEnabled, cdmEntitySessionEnabled);

		this.element = taxon;
		this.parentNode = parentNode;
		this.synonym = synonym;
		//this.namesInHomotypicGroup = namesInHomotypicalGroup;
	}


	/** {@inheritDoc} */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		
		UpdateResult result;
		try {
			result = CdmStore.getService(ITaxonService.class).changeSynonymToAcceptedTaxon(synonym.getUuid(),
			        element.getUuid(),
			        parentNode.getUuid(),
			        true);
			if (result.isAbort()){
				
				MessagingUtils.warningDialog(Messages.ChangeSynonymToAcceptedTaxonOperation_INCONSISTENT_DATA, getClass(), result.toString());
				return postExecute(null);
			}
			newNode = (TaxonNode) result.getCdmEntity();
		} catch (HomotypicalGroupChangeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		
		monitor.worked(20);
		//newTaxon = (Taxon) CdmStore.getService(ITaxonService.class).merge(newTaxon);
		//element.removeSynonym(synonym);


		/*if(namesInHomotypicGroup != null){
			Taxon taxon = newNode.getTaxon();
			for (TaxonNameBase synonymName : namesInHomotypicGroup){
				taxon.addHomotypicSynonymName(synonymName, null, null);

			}
		}*/
		//newNode = CdmStore.getService(ITaxonNodeService.class).merge(newNode);

		monitor.worked(40);

		return postExecute(newNode);
	}

	/** {@inheritDoc} */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return execute(monitor, info);
	}

	/** {@inheritDoc} */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		// TODO implement - biggest problem is that any window open for new taxon must be closed first
		MessagingUtils.warn(this.getClass(), Messages.ChangeSynonymToAcceptedTaxonOperation_NOT_IMPLEMENTED);

		return postExecute(element);
	}

}
