/**
* Copyright (C) 2013 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.view.derivate.searchFilter;

import java.util.List;
import java.util.UUID;

import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Widget;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.service.ITaxonService;
import eu.etaxonomy.cdm.api.service.config.FindOccurrencesConfigurator;
import eu.etaxonomy.cdm.api.service.config.FindOccurrencesConfigurator.AssignmentStatus;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationType;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.editor.view.derivate.DerivateView;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.SearchManager;
import eu.etaxonomy.taxeditor.ui.dialog.selection.SelectionDialogFactory;
import eu.etaxonomy.taxeditor.ui.mvc.combo.EnumTermComboController;
import eu.etaxonomy.taxeditor.view.derivateSearch.DerivateLabelProvider;

/**
 * Controller class for the {@link DerivateSearchComposite}.<br>
 * This controller contains the logic to search the data base for {@link SpecimenOrObservationBase}s
 * and handles GUI events.
 *
 */
public class DerivateSearchCompositeController implements Listener{

    private final DerivateSearchComposite derivateSearchComposite;
    private EnumTermComboController<SpecimenOrObservationType> comboSpecimenTypeController;
	private DerivateLabelProvider labelProvider;
	private Taxon selectedTaxon;
    private DerivateView derivativeEditor;

    public DerivateSearchCompositeController(Composite parent, DerivateView derivativeEditor) {
        this.derivateSearchComposite = new DerivateSearchComposite(parent, SWT.NONE);
        this.derivativeEditor = derivativeEditor;
        init();
    }

    private void init(){
        //derivate type combo
        comboSpecimenTypeController = new EnumTermComboController<SpecimenOrObservationType>(derivateSearchComposite.getComboDerivateType(), null, null, SpecimenOrObservationType.class, 0);
        comboSpecimenTypeController.setSelection(SpecimenOrObservationType.Unknown);

        //add listeners
        derivateSearchComposite.getSearchField().addListener(SWT.KeyDown, this);
        derivateSearchComposite.getButtonSearch().addListener(SWT.Selection, this);
        derivateSearchComposite.getBtnBrowseTaxa().addListener(SWT.Selection, this);
        derivateSearchComposite.getBtnClearTaxon().addListener(SWT.Selection, this);

    }

    private void searchDerivates(){
        if(derivativeEditor.isDirty()){
            boolean confirm = MessagingUtils.confirmDialog("Unsaved changes", "You have to save before performing this action. Save now?");
            if(confirm){
                derivativeEditor.save(new NullProgressMonitor());
            }
            else{
                return;
            }
        }
        String queryString = derivateSearchComposite.getSearchField().getText();
        SpecimenOrObservationType specimenType = comboSpecimenTypeController.getSelection();

        FindOccurrencesConfigurator config = new FindOccurrencesConfigurator();
        config.setAssociatedTaxonUuid(selectedTaxon!=null?selectedTaxon.getUuid():null);
        config.setSpecimenType(specimenType.equals(SpecimenOrObservationType.Unknown)?null:specimenType);//unknown := all types
        config.setTitleSearchString(queryString);
        if(selectedTaxon!=null){
            config.setSpecimenType(null);
        }
        switch (derivateSearchComposite.getComboTaxonAssignment().getSelectionIndex()) {
        case DerivateSearchComposite.ALL_SPECIMENS:
            config.setAssignmentStatus(AssignmentStatus.ALL_SPECIMENS);
            break;
        case DerivateSearchComposite.ASSIGNED_SPECIMENS:
            config.setAssignmentStatus(AssignmentStatus.ASSIGNED_SPECIMENS);
            break;
        case DerivateSearchComposite.UNASSIGNED_SPECIMENS:
            config.setAssignmentStatus(AssignmentStatus.UNASSIGNED_SPECIMENS);
            break;
        default:
            break;
        }
        if(derivativeEditor.getCdmEntitySession()!=null){
            derivativeEditor.getCdmEntitySession().dispose();
            derivativeEditor.getCdmEntitySession().bind();
        }

        SearchManager searchManager = new SearchManager();
        List<SpecimenOrObservationBase> occurrences = searchManager.findOccurrences(config, true);
        derivativeEditor.updateRootEntities(occurrences);
    }

    @Override
    public void handleEvent(Event event) {
        Widget eventSource = event.widget;
        if(eventSource == derivateSearchComposite.getButtonSearch()
                || (eventSource == derivateSearchComposite.getSearchField() && event.keyCode == SWT.CR)){
            searchDerivates();
        }
        else if(eventSource==derivateSearchComposite.getBtnBrowseTaxa()){
            selectedTaxon = SelectionDialogFactory.getSelectionFromDialog(Taxon.class, this.derivateSearchComposite.getShell(), derivativeEditor.getConversationHolder(), null);
            if(selectedTaxon!=null){
                derivateSearchComposite.getTextTaxonName().setText(selectedTaxon.getTitleCache());
            }
            derivateSearchComposite.getComboTaxonAssignment().select(DerivateSearchComposite.ASSIGNED_SPECIMENS);
            derivateSearchComposite.getComboTaxonAssignment().setEnabled(false);
            derivateSearchComposite.getLbltaxonAssignment().setEnabled(false);
        }
        else if(eventSource==derivateSearchComposite.getBtnClearTaxon()){
            selectedTaxon = null;
            derivateSearchComposite.getTextTaxonName().setText(""); //$NON-NLS-1$
            derivateSearchComposite.getComboTaxonAssignment().select(DerivateSearchComposite.ALL_SPECIMENS);
            derivateSearchComposite.getComboTaxonAssignment().setEnabled(true);
            derivateSearchComposite.getLbltaxonAssignment().setEnabled(true);
        }
    }

    /**
     * Sets enable status of all input fields
     * @param enabled
     */
    public void setEnabled(boolean enabled) {
        if(!derivateSearchComposite.isDisposed()){
            GridData layoutData = (GridData) derivateSearchComposite.getLayoutData();
            layoutData.exclude = !enabled;
            derivateSearchComposite.setVisible(enabled);
            derivateSearchComposite.getParent().layout(true);
        }
    }

    public void setLayoutData(Object layoutData){
        derivateSearchComposite.setLayoutData(layoutData);
    }

    /**
     * Checks whether or not the search criteria are set to their default value.<br>
     * This does not consider the title search field.
     * @return <code>true</code> if search criteria are set to default.
     */
    public boolean isDefaultSearch(){
        boolean isDefault = true;
        isDefault &= comboSpecimenTypeController.getSelection()==SpecimenOrObservationType.Unknown;
        isDefault &= derivateSearchComposite.getComboTaxonAssignment().getSelectionIndex()==DerivateSearchComposite.ALL_SPECIMENS;
        isDefault &= selectedTaxon==null;
        return isDefault;
    }
    /**
     * Resets all input fields
     */
    public void reset(){
        if(!derivateSearchComposite.isDisposed()){
            derivateSearchComposite.getSearchField().setText(""); //$NON-NLS-1$
            derivateSearchComposite.getComboDerivateType().deselectAll();
            comboSpecimenTypeController.setSelection(SpecimenOrObservationType.Unknown);
        }
        selectedTaxon = null;
    }

    public void setConversation(ConversationHolder conversationHolder) {
        labelProvider.setConversation(conversationHolder);
    }

    public void setTaxonFilter(UUID taxonUuid) {
        TaxonBase taxonBase = CdmStore.getService(ITaxonService.class).load(taxonUuid);
        if(taxonBase!=null && taxonBase.isInstanceOf(Taxon.class)){
            selectedTaxon = HibernateProxyHelper.deproxy(taxonBase, Taxon.class);
            derivateSearchComposite.getTextTaxonName().setText(taxonBase.getTitleCache());
            derivateSearchComposite.getLbltaxonAssignment().setEnabled(false);
            derivateSearchComposite.getComboTaxonAssignment().select(DerivateSearchComposite.ASSIGNED_SPECIMENS);
            derivateSearchComposite.getComboTaxonAssignment().setEnabled(false);
        }
    }

    public void setFocusOnSearchField(){
    	derivateSearchComposite.getSearchField().setFocus();
    }

}
