/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.descriptive.handler;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.api.service.config.MediaDeletionConfigurator;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.SpecimenDescription;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.description.TaxonNameDescription;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.editor.view.descriptive.operation.DeleteDescriptionElementOperation;
import eu.etaxonomy.taxeditor.editor.view.descriptive.operation.DeleteSpecimenDescriptionOperation;
import eu.etaxonomy.taxeditor.editor.view.descriptive.operation.DeleteTaxonDescriptionOperation;
import eu.etaxonomy.taxeditor.editor.view.media.operation.DeleteMediaOperation;
import eu.etaxonomy.taxeditor.editor.view.media.operation.RemoveImageFromDescriptionElementOperation;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.FeatureNodeContainer;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled;
import eu.etaxonomy.taxeditor.ui.dialog.configurator.deleteConfigurator.DeleteConfiguratorDialog;

/**
 * <p>DeleteDescriptionHandler class.</p>
 *
 * @author n.hoffmann
 * @created Jun 22, 2010
 * @version 1.0
 */
public class DeleteHandler extends AbstractHandler {

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	/** {@inheritDoc} */
	@Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
		IStructuredSelection selection = (IStructuredSelection) HandlerUtil.getActiveSite(event).getSelectionProvider().getSelection();

		IWorkbenchPart part = HandlerUtil.getActivePart(event);
		IPostOperationEnabled postOperationEnabled = (part instanceof IPostOperationEnabled) ? (IPostOperationEnabled) part : null;
		ICdmEntitySessionEnabled cdmEntitySessionEnabled = (part instanceof ICdmEntitySessionEnabled) ? (ICdmEntitySessionEnabled) part : null;

		try {
			String label = event.getCommand().getName();

			IUndoContext undoContext = EditorUtil.getUndoContext();

			List<AbstractPostOperation<?>> operations = new ArrayList<AbstractPostOperation<?>>();

			for(Object object : selection.toArray()){

				// TaxonDescription
				if(object instanceof TaxonDescription){
					operations.add(new DeleteTaxonDescriptionOperation(label, undoContext, (TaxonDescription) object, postOperationEnabled, cdmEntitySessionEnabled));
				}
				else if(object instanceof SpecimenDescription){
				    operations.add(new DeleteSpecimenDescriptionOperation(label, undoContext, (SpecimenDescription) object, postOperationEnabled, cdmEntitySessionEnabled)) ;
				}
				// DescriptionElementBase
				else if(object instanceof DescriptionElementBase){
					operations.add(new DeleteDescriptionElementOperation(label, undoContext, (DescriptionElementBase) object, postOperationEnabled, cdmEntitySessionEnabled));
				}
				else if(object instanceof FeatureNodeContainer){
					List<DescriptionElementBase> descriptions = ((FeatureNodeContainer) object).getDescriptionElementsForEntireBranch();

					for(DescriptionElementBase description : descriptions){
						operations.add(new DeleteDescriptionElementOperation(label, undoContext, description, postOperationEnabled, cdmEntitySessionEnabled));
					}
				}
				// Media
				else if(object instanceof Media){
					TreeSelection treeSelection = (TreeSelection) selection;

					TreePath[] path = treeSelection.getPathsFor(object);

					DescriptionBase<?> imageGallery = (DescriptionBase<?>) path[0].getFirstSegment();

					// TODO use undo context specific to editor
                    MediaDeletionConfigurator config = new MediaDeletionConfigurator();
                    config.setDeleteFromDescription(true);
                    config.setOnlyRemoveFromGallery(false);
                    
                    if (imageGallery instanceof SpecimenDescription){
                    	config.setDeleteFrom(((SpecimenDescription)imageGallery).getDescribedSpecimenOrObservation());
                    } else if (imageGallery instanceof TaxonDescription){
                    	config.setDeleteFrom(((TaxonDescription)imageGallery).getTaxon());
                    }else if (imageGallery instanceof TaxonNameDescription){
                    	config.setDeleteFrom(((TaxonNameDescription)imageGallery).getTaxonName());
                    }
                    	

                    if (!DeleteConfiguratorDialog.openConfirmWithConfigurator(config, HandlerUtil.getActiveShell(event), Messages.DeleteHandler_CONFIRM_DELETION,  Messages.DeleteHandler_CONFIRM_DELETION_MESSAGE)){
                        return null;
                    }



                    if (config.isOnlyRemoveFromGallery() || ((Media)object).getId() == 0){
                        operations.add(new RemoveImageFromDescriptionElementOperation(label, undoContext,  (Media) object, imageGallery, postOperationEnabled));
                    }else{
                        operations.add(new DeleteMediaOperation(label, undoContext, imageGallery, (Media) object, config, postOperationEnabled));
                    }



				}
				else{
					MessagingUtils.error(getClass(), Messages.DeleteHandler_INVALID_SELECTION, null);
				}
			}

			// execute all cumulated operations
			for(AbstractPostOperation<?> operation : operations){
				AbstractUtility.executeOperation(operation);
			}

		} catch (NotDefinedException e) {
			MessagingUtils.warn(getClass(), "Command name not set."); //$NON-NLS-1$
		}


		return null;
	}
}
